// ******************************************
// ** USB Mouse for the ZX81 & Other 8bits **
// **                                      **
// ** zx81keyboardadventure.com 2018       **
// ******************************************
// ** Copyright FREE for all to use        **
// ** If you like it use and improve it,   **
// ** share it and post it.                **
// ******************************************

#include <hidboot.h>
#include <usbhub.h>
#include "ZeaMouse.h"

// Satisfy the IDE, which needs to see the include statement in the ino too.
#ifdef dobogusinclude
#include <spi4teensy3.h>
#include <SPI.h>
#endif

const byte gcbDebounce = 6;

// **********************
// *** USB Host Setup ***
// **********************

USB     Usb;
USBHub	Hub(&Usb);
HIDBoot	<USB_HID_PROTOCOL_MOUSE>    HidMouse(&Usb);
uint32_t next_time;

// **********************************
// *** Global Variables & Classes ***
// **********************************
MouseRptParser gcMouseAction;

const byte gbJoyPinsOut[5] = {6,A1,7,8,A0}; // Up, Down, Left, Right, Fire
float gfMouseDebounce[4] = {gcbDebounce,gcbDebounce,gcbDebounce,gcbDebounce};

void setup()
{
	Serial.begin( 115200 );
	#if !defined(__MIPSEL__)
	while (!Serial); // Wait for serial port to connect - used on Leonardo, Teensy and other boards with built-in USB CDC serial connection
	#endif
	Serial.println("Start");

	if (Usb.Init() == -1)
		Serial.println("OSC did not start.");

	delay( 200 );

	next_time = millis() + 5000;

	HidMouse.SetReportParser(0, &gcMouseAction);

	byte bCount;

	for(bCount = 0 ; bCount < 5; bCount ++){
		pinMode(gbJoyPinsOut[bCount], OUTPUT);
		digitalWrite(gbJoyPinsOut[bCount], LOW);
	}

	pinMode(LED_BUILTIN,OUTPUT);
	digitalWrite(LED_BUILTIN, LOW);
}

void loop()
{
	byte bCount;
	Usb.Task();


	// Read XY movements of mouse
	for(bCount = 0 ; bCount < 4; bCount ++){

	digitalWrite(gbJoyPinsOut[bCount], LOW);

		if(millis() - gcMouseAction.plMovedLastMills[bCount] < gfMouseDebounce[bCount]){
			digitalWrite(gbJoyPinsOut[bCount], HIGH);
			// Mouse exceleration
			if (gfMouseDebounce[bCount] < 20 ) {
				// Higher than 22 freaks out the ZX81
				gfMouseDebounce[bCount]+=0.125;
				Serial.println(gfMouseDebounce[bCount]);
			}
		} else {
			gfMouseDebounce[bCount]=gcbDebounce;
		}
	}

	// Read Button Press. Mouse button has higher priority in this case
	// as the mouse button reports pressed until released.
	if(gcMouseAction.plMovedLastMills[4]){
		digitalWrite(gbJoyPinsOut[4], HIGH);
	} else {
		digitalWrite(gbJoyPinsOut[4], LOW);
	}
}







